#include <math.h>
#include <stdio.h>
#include "../src/grind.h"

static FILE *ps;
static double scale = 400;
static double margin = 0.3;
static int twod = 1;

int triplt(double,double,double,double*, double*);
int setProjection();

int gpsCall(const _grcall *tmp_grcall) {
  /* _line_elem *line; */
  static double cpos[3] = {0.};
  double x,y,size;
  /*  double *tmp_coord; */
  int c,type;
  char *str;
  double *coord;

  switch (type = tmp_grcall->type) {
  case CLEAR:
    break;
  case TWOD:
    twod = 1;
    break;
  case THREED:
    twod = 0;
    setProjection();
    break;
  case GR_POINT:
  case MOVE:
  case DRAW:
    coord = (double*)(tmp_grcall->val);
    x = cpos[0] = coord[0];
    y = cpos[1] = coord[1];
    cpos[2] = coord[2];
    if (!twod) triplt(cpos[0],cpos[1],cpos[2],&x,&y);
    if (type != DRAW)
      fprintf(ps,"%6.2f %6.2f m\n",x*scale,y*scale);
    if (type != MOVE)
      fprintf(ps,"%6.2f %6.2f d\n",x*scale,y*scale);
    break;
  case ORECT:
  case FRECT:
    coord = (double*)(tmp_grcall->val);
    x = cpos[0] = coord[0];
    y = cpos[1] = coord[1];
    cpos[2] = coord[2];
    if (!twod) triplt(cpos[0],cpos[1],cpos[2],&x,&y);
    size = scale/50.;
    x = x*scale - size/2.;
    y = y*scale - size/2.;
    if (ORECT == type)
      fprintf(ps,"%6.2f %6.2f m %6.2f %6.2f d %6.2f %6.2f d\n%6.2f %6.2f d %6.2f %6.2f d\n",
	      x,y,x+size,y,x+size,y+size,x,y+size,x,y);
    else
      fprintf(ps,"%6.2f %6.2f %6.2f %6.2f r\n",x,y,size,size);
    break;
  case TEXT:
  case LTEXT:
  case RTEXT:
    str = (char*)(tmp_grcall->val);
    fprintf(ps,"(%s) courier\n",str);
    break;  
  case COLOR:
    c = colorWrap(*((int*)(tmp_grcall->val)));
    if (0 == c) c = 1;
    if (1 == c) c = 0;
    fprintf(ps,"1 setlinewidth\n %4.2f %4.2f %4.2f rgb\n",
	    colormap[c][0],colormap[c][1],colormap[c][2]);
    break;
  default:
    fprintf(stderr,"error: trying to draw unknown primitive: %d\n", 
	    tmp_grcall->type);
    break;
  }
  return 1;
}

int gps(char *name) {
  extern _grcall *first_grcall;  /*  GRIND POINTERS !!! */
  extern _grcall *last_grcall;   /*  GRIND POINTERS !!! */
  const _grcall *tmp = first_grcall;

  ps = fopen(name,"w");
  printf("Opening %s\n",name);
  fprintf(ps,"%%!PS-Adobe-2.0\n");
  fprintf(ps,"%%%%BoundingBox: 0 0 %4.0f %4.0f\n", 
	  scale*(1+margin), scale*(1+margin));
  fprintf(ps,"1 1 scale\n");
  fprintf(ps,"/courier {/Courier findfont 14 scalefont setfont show}def\n");
  fprintf(ps,"/italic {/Courier-Oblique findfont 14 scalefont setfont show}def\n");
  fprintf(ps,"/symbol {/Symbol findfont 14 scalefont setfont show}def\n");
  fprintf(ps,"/l {lineto}def\n");
  fprintf(ps,"/g {setgray}def\n");
  fprintf(ps,"/rgb {setrgbcolor}def\n");
  fprintf(ps,"/f {closepath fill}def\n");
  fprintf(ps,"/m {newpath moveto}def\n");
  fprintf(ps,"/d {lineto currentpoint stroke moveto}def\n");
  fprintf(ps,"/r{/h exch def /w exch def\n");
  fprintf(ps,"   /y exch def /x exch def\n");
  fprintf(ps,"   newpath x y moveto w 0 rlineto\n");
  fprintf(ps,"   0 h rlineto w neg 0 rlineto\n");
  fprintf(ps,"   closepath fill}def\n");
  fprintf(ps,"1 setlinecap newpath 0 0 moveto\n");
  fprintf(ps,"%4.1f %4.1f translate\n", scale*margin, scale*margin);

  gpsCall(tmp);
  while(tmp != last_grcall) {
    tmp = tmp->next;
    gpsCall(tmp);
  } 
  fprintf(ps,"showpage\n");
  fclose(ps);
  return 1;
}

